@@ -474,7 +474,7 @@ bool AlnRes::matchesRef(
 	rdseq.clear();
 	rdseq = rd.patFw;
 	if(!fw) {
-		rdseq.reverseComp(false);
+		rdseq.reverseComp();
 	}
 	assert_eq(rdrows_, rdseq.length());
 	// rdseq is the nucleotide sequence from upstream to downstream on the
@@ -930,6 +930,7 @@ void RedundantAlns::add(const AlnRes& res) {
 	assert(!cells_.empty());
 	TRefOff left = res.refoff(), right;
 	const size_t len = res.readExtentRows();
+        const size_t alignmentStart = res.trimmedLeft(true);
 	if(!res.fw()) {
 		const_cast<AlnRes&>(res).invertEdits();
 	}
@@ -937,7 +938,7 @@ void RedundantAlns::add(const AlnRes& res) {
 	size_t nedidx = 0;
 	assert_leq(len, cells_.size());
 	// For each row...
-	for(size_t i = 0; i < len; i++) {
+	for(size_t i = alignmentStart; i < alignmentStart + len; i++) {
 		size_t diff = 1;  // amount to shift to right for next round
 		right = left + 1;
 		while(nedidx < ned.size() && ned[nedidx].pos == i) {
@@ -947,7 +948,7 @@ void RedundantAlns::add(const AlnRes& res) {
 			}
 			nedidx++;
 		}
-		if(i < len - 1) {
+		if(i < alignmentStart + len - 1) {
 			// See how many inserts there are before the next read
 			// character
 			size_t nedidx_next = nedidx;
@@ -980,6 +981,7 @@ bool RedundantAlns::overlap(const AlnRes& res) {
 	assert(!cells_.empty());
 	TRefOff left = res.refoff(), right;
 	const size_t len = res.readExtentRows();
+        const size_t alignmentStart = res.trimmedLeft(true);
 	if(!res.fw()) {
 		const_cast<AlnRes&>(res).invertEdits();
 	}
@@ -988,7 +990,7 @@ bool RedundantAlns::overlap(const AlnRes& res) {
 	// For each row...
 	bool olap = false;
 	assert_leq(len, cells_.size());
-	for(size_t i = 0; i < len; i++) {
+	for(size_t i = alignmentStart; i < alignmentStart + len; i++) {
 		size_t diff = 1;  // amount to shift to right for next round
 		right = left + 1;
 		while(nedidx < ned.size() && ned[nedidx].pos == i) {
@@ -998,7 +1000,7 @@ bool RedundantAlns::overlap(const AlnRes& res) {
 			}
 			nedidx++;
 		}
-		if(i < len - 1) {
+		if(i < alignmentStart + len - 1) {
 			// See how many inserts there are before the next read
 			// character
 			size_t nedidx_next = nedidx;
@@ -1055,12 +1057,15 @@ void AlnSetSumm::init(
 	bestPaired.invalidate(); secbestPaired.invalidate();
 	bool paired = (rs1 != NULL && rs2 != NULL);
 	szs[0] = szs[1] = 0;
+	// Set bestPaired and secbestPaired
 	if(paired) {
 		// Paired alignments
 		assert_eq(rs1->size(), rs2->size());
 		szs[0] = szs[1] = rs1->size();
 		assert_gt(szs[0], 0);
 		for(size_t i = 0; i < rs1->size(); i++) {
+			// Combine mate scores into a concordant alignment score by
+			// summing them
 			AlnScore sc = (*rs1)[i].score() + (*rs2)[i].score();
 			if(sc > bestPaired) {
 				secbestPaired = bestPaired;
@@ -1073,14 +1078,13 @@ void AlnSetSumm::init(
 			}
 		}
 	}
+	// Set best[] and secbest[]
 	for(int j = 0; j < 2; j++) {
 		const EList<AlnRes>* rs = (j == 0 ? rs1u : rs2u);
 		if(rs == NULL) {
 			continue;
 		}
-		assert(rs != NULL);
 		szs[j] = rs->size();
-		//assert_gt(szs[j], 0);
 		for(size_t i = 0; i < rs->size(); i++) {
 			AlnScore sc = (*rs)[i].score();
 			if(sc > best[j]) {
