@@ -38,6 +38,7 @@
 #include "scoring.h"
 #include "mem_ids.h"
 #include "simple_func.h"
+#include "btypes.h"
 
 /**
  * A constraint to apply to an alignment zone, or to an overall
@@ -490,8 +491,8 @@ struct EEHit {
 	}
 	
 	void init(
-		uint32_t top_,
-		uint32_t bot_,
+		TIndexOffU top_,
+		TIndexOffU bot_,
 		const Edit* e1_,
 		const Edit* e2_,
 		bool fw_,
@@ -566,7 +567,7 @@ struct EEHit {
 	/**
 	 * Return the size of the alignments SA range.s
 	 */
-	uint32_t size() const { return bot - top; }
+	TIndexOffU size() const { return bot - top; }
 	
 #ifndef NDEBUG
 	/**
@@ -584,8 +585,8 @@ struct EEHit {
 	}
 #endif
 	
-	uint32_t top;
-	uint32_t bot;
+	TIndexOffU top;
+	TIndexOffU bot;
 	Edit     e1;
 	Edit     e2;
 	bool     fw;
@@ -669,6 +670,11 @@ public:
 		numRanges_ += qv.numRanges();
 		if(qv.numRanges() > 0) {
 			nonzTot_++;
+			if(qv.numRanges() == 1 and qv.numElts() == 1) {
+				uniTot_++;
+			} else {
+				repTot_++;
+			}
 		}
 		assert(repOk(&ac));
 	}
@@ -723,6 +729,8 @@ public:
 		seqFw_.clear();
 		seqRc_.clear();
 		nonzTot_ = 0;
+		uniTot_ = 0;
+		repTot_ = 0;
 		nonzFw_ = 0;
 		nonzRc_ = 0;
 		numOffs_ = 0;
@@ -810,6 +818,20 @@ public:
 	float averageHitsPerSeed() const {
 		return (float)numElts_ / (float)nonzTot_;
 	}
+
+	/**
+	 * Return fraction of seeds that align uniquely.
+	 */
+	size_t numUniqueSeeds() const {
+		return uniTot_;
+	}
+
+	/**
+	 * Return fraction of seeds that align repetitively.
+	 */
+	size_t numRepeatSeeds() const {
+		return repTot_;
+	}
 	
 	/**
 	 * Return median of all the non-zero per-seed # hits
@@ -974,43 +996,64 @@ public:
 	 * that are more unique) will be tried first and QVals with more elements
 	 * (i.e. seed sequences
 	 */
-	void rankSeedHits(RandomSource& rnd) {
-		while(rankOffs_.size() < nonzTot_) {
-			uint32_t minsz = 0xffffffff;
-			uint32_t minidx = 0;
-			bool minfw = true;
-			// Rank seed-hit positions in ascending order by number of elements
-			// in all BW ranges
-			bool rb = rnd.nextBool();
-			assert(rb == 0 || rb == 1);
-			for(int fwi = 0; fwi <= 1; fwi++) {
-				bool fw = (fwi == (rb ? 1 : 0));
-				EList<QVal>& rrs = (fw ? hitsFw_ : hitsRc_);
-				EList<bool>& sorted = (fw ? sortedFw_ : sortedRc_);
-				uint32_t i = (rnd.nextU32() % (uint32_t)numOffs_);
-				for(uint32_t ii = 0; ii < numOffs_; ii++) {
-					if(rrs[i].valid() &&         // valid QVal
-					   rrs[i].numElts() > 0 &&   // non-empty
-					   !sorted[i] &&             // not already sorted
-					   rrs[i].numElts() < minsz) // least elts so far?
-					{
-						minsz = rrs[i].numElts();
-						minidx = i;
-						minfw = (fw == 1);
-					}
-					if((++i) == numOffs_) {
-						i = 0;
+	void rankSeedHits(RandomSource& rnd, bool all) {
+		if(all) {
+			for(uint32_t i = 1; i < numOffs_; i++) {
+				for(int fwi = 0; fwi <= 1; fwi++) {
+					bool fw = fwi == 0;
+					EList<QVal>& rrs = (fw ? hitsFw_ : hitsRc_);
+					if(rrs[i].valid() && rrs[i].numElts() > 0) {
+						rankOffs_.push_back(i);
+						rankFws_.push_back(fw);
 					}
 				}
 			}
-			assert_neq(0xffffffff, minsz);
-			if(minfw) {
-				sortedFw_[minidx] = true;
-			} else {
-				sortedRc_[minidx] = true;
+			if(hitsFw_[0].valid() && hitsFw_[0].numElts() > 0) {
+				rankOffs_.push_back(0);
+				rankFws_.push_back(true);
+			}
+			if(hitsRc_[0].valid() && hitsRc_[0].numElts() > 0) {
+				rankOffs_.push_back(0);
+				rankFws_.push_back(false);
+			}
+		} else {
+			while(rankOffs_.size() < nonzTot_) {
+				TIndexOffU minsz = MAX_U32;
+				uint32_t minidx = 0;
+				bool minfw = true;
+				// Rank seed-hit positions in ascending order by number of elements
+				// in all BW ranges
+				bool rb = rnd.nextBool();
+				assert(rb == 0 || rb == 1);
+				for(int fwi = 0; fwi <= 1; fwi++) {
+					bool fw = (fwi == (rb ? 1 : 0));
+					EList<QVal>& rrs = (fw ? hitsFw_ : hitsRc_);
+					EList<bool>& sorted = (fw ? sortedFw_ : sortedRc_);
+					uint32_t i = (rnd.nextU32() % (uint32_t)numOffs_);
+					for(uint32_t ii = 0; ii < numOffs_; ii++) {
+						if(rrs[i].valid() &&         // valid QVal
+						   rrs[i].numElts() > 0 &&   // non-empty
+						   !sorted[i] &&             // not already sorted
+						   rrs[i].numElts() < minsz) // least elts so far?
+						{
+							minsz = rrs[i].numElts();
+							minidx = i;
+							minfw = (fw == 1);
+						}
+						if((++i) == numOffs_) {
+							i = 0;
+						}
+					}
+				}
+				assert_neq(MAX_U32, minsz);
+				if(minfw) {
+					sortedFw_[minidx] = true;
+				} else {
+					sortedRc_[minidx] = true;
+				}
+				rankOffs_.push_back(minidx);
+				rankFws_.push_back(minfw);
 			}
-			rankOffs_.push_back(minidx);
-			rankFws_.push_back(minfw);
 		}
 		assert_eq(rankOffs_.size(), rankFws_.size());
 		sorted_ = true;
@@ -1183,8 +1226,8 @@ public:
 	 * Add an end-to-end 1-mismatch alignment.
 	 */
 	void add1mmEe(
-		uint32_t top,
-		uint32_t bot,
+		TIndexOffU top,
+		TIndexOffU bot,
 		const Edit* e1,
 		const Edit* e2,
 		bool fw,
@@ -1199,8 +1242,8 @@ public:
 	 * Add an end-to-end exact alignment.
 	 */
 	void addExactEeFw(
-		uint32_t top,
-		uint32_t bot,
+		TIndexOffU top,
+		TIndexOffU bot,
 		const Edit* e1,
 		const Edit* e2,
 		bool fw,
@@ -1213,8 +1256,8 @@ public:
 	 * Add an end-to-end exact alignment.
 	 */
 	void addExactEeRc(
-		uint32_t top,
-		uint32_t bot,
+		TIndexOffU top,
+		TIndexOffU bot,
 		const Edit* e1,
 		const Edit* e2,
 		bool fw,
@@ -1285,6 +1328,8 @@ protected:
 	EList<bool>         sortedFw_;    // true iff fw QVal was sorted/ranked
 	EList<bool>         sortedRc_;    // true iff rc QVal was sorted/ranked
 	size_t              nonzTot_;     // # offsets with non-zero size
+	size_t              uniTot_;      // # offsets unique hit
+	size_t              repTot_;      // # offsets repetitive hit
 	size_t              nonzFw_;      // # offsets into fw read with non-0 size
 	size_t              nonzRc_;      // # offsets into rc read with non-0 size
 	size_t              numRanges_;   // # ranges added
@@ -1340,6 +1385,8 @@ struct SeedSearchMetrics {
 	void merge(const SeedSearchMetrics& m, bool getLock = false) {
         ThreadSafe ts(&mutex_m, getLock);
 		seedsearch   += m.seedsearch;
+		nrange       += m.nrange;
+		nelt         += m.nelt;
 		possearch    += m.possearch;
 		intrahit     += m.intrahit;
 		interhit     += m.interhit;
@@ -1357,6 +1404,8 @@ struct SeedSearchMetrics {
 	 */
 	void reset() {
 		seedsearch =
+		nrange =
+		nelt =
 		possearch =
 		intrahit =
 		interhit =
@@ -1370,6 +1419,8 @@ struct SeedSearchMetrics {
 	}
 
 	uint64_t seedsearch;   // # times we executed strategy in InstantiatedSeed
+	uint64_t nrange;       // # ranges found
+	uint64_t nelt;         // # range elements found
 	uint64_t possearch;    // # offsets where aligner executed >= 1 strategy
 	uint64_t intrahit;     // # offsets where current-read cache gave answer
 	uint64_t interhit;     // # offsets where across-read cache gave answer
@@ -1449,10 +1500,10 @@ public:
 		size_t             dep,
 		size_t             len,
 		bool               do1mm,
-		uint32_t           topfw,
-		uint32_t           botfw,
-		uint32_t           topbw,
-		uint32_t           botbw);
+		TIndexOffU           topfw,
+		TIndexOffU           botfw,
+		TIndexOffU           topbw,
+		TIndexOffU           botbw);
 
 	/**
 	 * Do an exact-matching sweet to establish a lower bound on number of edits
@@ -1498,10 +1549,10 @@ protected:
 	 * calling reportHit().
 	 */
 	bool extendAndReportHit(
-		uint32_t topf,                     // top in BWT
-		uint32_t botf,                     // bot in BWT
-		uint32_t topb,                     // top in BWT'
-		uint32_t botb,                     // bot in BWT'
+		TIndexOffU topf,                     // top in BWT
+		TIndexOffU botf,                     // bot in BWT
+		TIndexOffU topb,                     // top in BWT'
+		TIndexOffU botb,                     // bot in BWT'
 		uint16_t len,                      // length of hit
 		DoublyLinkedList<Edit> *prevEdit); // previous edit
 
@@ -1510,10 +1561,10 @@ protected:
 	 * false if the hit could not be reported because of, e.g., cache exhaustion.
 	 */
 	bool reportHit(
-		uint32_t topf,         // top in BWT
-		uint32_t botf,         // bot in BWT
-		uint32_t topb,         // top in BWT'
-		uint32_t botb,         // bot in BWT'
+		TIndexOffU topf,         // top in BWT
+		TIndexOffU botf,         // bot in BWT
+		TIndexOffU topb,         // top in BWT'
+		TIndexOffU botb,         // bot in BWT'
 		uint16_t len,          // length of hit
 		DoublyLinkedList<Edit> *prevEdit);  // previous edit
 	
@@ -1528,10 +1579,10 @@ protected:
 	bool searchSeedBi(
 		int step,              // depth into steps_[] array
 		int depth,             // recursion depth
-		uint32_t topf,         // top in BWT
-		uint32_t botf,         // bot in BWT
-		uint32_t topb,         // top in BWT'
-		uint32_t botb,         // bot in BWT'
+		TIndexOffU topf,         // top in BWT
+		TIndexOffU botf,         // bot in BWT
+		TIndexOffU topb,         // top in BWT'
+		TIndexOffU botb,         // bot in BWT'
 		SideLocus tloc,        // locus for top (perhaps unititialized)
 		SideLocus bloc,        // locus for bot (perhaps unititialized)
 		Constraint c0,         // constraints to enforce in seed zone 0
@@ -1546,10 +1597,10 @@ protected:
 	inline void nextLocsBi(
 		SideLocus& tloc,            // top locus
 		SideLocus& bloc,            // bot locus
-		uint32_t topf,              // top in BWT
-		uint32_t botf,              // bot in BWT
-		uint32_t topb,              // top in BWT'
-		uint32_t botb,              // bot in BWT'
+		TIndexOffU topf,              // top in BWT
+		TIndexOffU botf,              // bot in BWT
+		TIndexOffU topb,              // top in BWT'
+		TIndexOffU botb,              // bot in BWT'
 		int step);                  // step to get ready for
 	
 	// Following are set in searchAllSeeds then used by searchSeed()
@@ -1589,8 +1640,8 @@ protected:
 }
 
 #define SANITY_CHECK_4TUP(t, b, tp, bp) { \
-	ASSERT_ONLY(uint32_t tot = (b[0]-t[0])+(b[1]-t[1])+(b[2]-t[2])+(b[3]-t[3])); \
-	ASSERT_ONLY(uint32_t totp = (bp[0]-tp[0])+(bp[1]-tp[1])+(bp[2]-tp[2])+(bp[3]-tp[3])); \
+	ASSERT_ONLY(TIndexOffU tot = (b[0]-t[0])+(b[1]-t[1])+(b[2]-t[2])+(b[3]-t[3])); \
+	ASSERT_ONLY(TIndexOffU totp = (bp[0]-tp[0])+(bp[1]-tp[1])+(bp[2]-tp[2])+(bp[3]-tp[3])); \
 	assert_eq(tot, totp); \
 }
 
