@@ -55,9 +55,9 @@
 #include <limits>
 #include "aligner_sw.h"
 
-static const size_t NBYTES_PER_REG  = 16;
+//static const size_t NBYTES_PER_REG  = 16;
 static const size_t NWORDS_PER_REG  = 16;
-static const size_t NBITS_PER_WORD  = 8;
+//static const size_t NBITS_PER_WORD  = 8;
 static const size_t NBYTES_PER_WORD = 1;
 
 // In local mode, we start low (0) and go high (255).  Factoring in a query
@@ -2011,7 +2011,6 @@ bool SwAligner::backtraceNucleotidesLocalSseU8(
 			// it and add a nucleotide mismatch.
 			case SW_BT_OALL_DIAG: {
 				assert_gt(row, 0); assert_gt(col, 0);
-				// Check for color mismatch
 				int readC = (*rd_)[row];
 				int refNmask = (int)rf_[rfi_+col];
 				assert_gt(refNmask, 0);
