@@ -20,6 +20,11 @@
 #ifndef DIFF_SAMPLE_H_
 #define DIFF_SAMPLE_H_
 
+#ifdef WITH_TBB
+#include <tbb/tbb.h>
+#include <tbb/task_group.h>
+#endif
+
 #include <stdint.h>
 #include <string.h>
 #include "assert_helpers.h"
@@ -28,6 +33,7 @@
 #include "ds.h"
 #include "mem_ids.h"
 #include "ls.h"
+#include "btypes.h"
 
 using namespace std;
 
@@ -452,7 +458,7 @@ public:
 		_isaPrime(),
 		_dInv(),
 		_log2v(myLog2(_v)),
-		_vmask(0xffffffff << _log2v),
+		_vmask(OFF_MASK << _log2v),
 		_logger(__logger)
 	{
 		assert_gt(_d, 0);
@@ -478,15 +484,15 @@ public:
 		size_t sPrimeSz = (len / v) * ds.size();
 		// sPrime, sPrimeOrder, _isaPrime all exist in memory at
 		// once and that's the peak
-		AutoArray<uint32_t> aa(sPrimeSz * 3 + (1024 * 1024 /*out of caution*/), EBWT_CAT);
+		AutoArray<TIndexOffU> aa(sPrimeSz * 3 + (1024 * 1024 /*out of caution*/), EBWT_CAT);
 		return sPrimeSz * 4; // sPrime array
 	}
 
 	uint32_t v() const                   { return _v; }
 	uint32_t log2v() const               { return _log2v; }
 	uint32_t vmask() const               { return _vmask; }
-	uint32_t modv(uint32_t i) const      { return i & ~_vmask; }
-	uint32_t divv(uint32_t i) const      { return i >> _log2v; }
+	uint32_t modv(TIndexOffU i) const    { return (uint32_t)(i & ~_vmask); }
+	TIndexOffU divv(TIndexOffU i) const  { return i >> _log2v; }
 	uint32_t d() const                   { return _d; }
 	bool verbose() const                 { return _verbose; }
 	bool sanityCheck() const             { return _sanity; }
@@ -495,11 +501,11 @@ public:
 	const EList<uint32_t>& dmap() const  { return _dmap; }
 	ostream& log() const                 { return _logger; }
 
-	void     build();
-	uint32_t tieBreakOff(uint32_t i, uint32_t j) const;
-	int64_t  breakTie(uint32_t i, uint32_t j) const;
-	bool     isCovered(uint32_t i) const;
-	uint32_t rank(uint32_t i) const;
+	void     build(int nthreads);
+	uint32_t tieBreakOff(TIndexOffU i, TIndexOffU j) const;
+	int64_t  breakTie(TIndexOffU i, TIndexOffU j) const;
+	bool     isCovered(TIndexOffU i) const;
+	TIndexOffU rank(TIndexOffU i) const;
 
 	/**
 	 * Print out the suffix array such that every sample offset has its
@@ -522,7 +528,7 @@ public:
 private:
 
 	void doBuiltSanityCheck() const;
-	void buildSPrime(EList<uint32_t>& sPrime, size_t padding);
+	void buildSPrime(EList<TIndexOffU>& sPrime, size_t padding);
 
 	bool built() const {
 		return _isaPrime.size() > 0;
@@ -542,11 +548,11 @@ private:
 	EList<uint32_t>  _ds;       // samples: idx -> d
 	EList<uint32_t>  _dmap;     // delta map
 	uint32_t         _d;        // |D| - size of sample
-	EList<uint32_t>  _doffs;    // offsets into sPrime/isaPrime for each d idx
-	EList<uint32_t>  _isaPrime; // ISA' array
+	EList<TIndexOffU>  _doffs;    // offsets into sPrime/isaPrime for each d idx
+	EList<TIndexOffU>  _isaPrime; // ISA' array
 	EList<uint32_t>  _dInv;     // Map from d -> idx
 	uint32_t         _log2v;
-	uint32_t         _vmask;
+	TIndexOffU         _vmask;
 	ostream&         _logger;
 };
 
@@ -559,17 +565,17 @@ void DifferenceCoverSample<TStr>::doBuiltSanityCheck() const {
 	uint32_t v = this->v();
 	assert(built());
 	VMSG_NL("  Doing sanity check");
-	uint32_t added = 0;
-	EList<uint32_t> sorted;
+	TIndexOffU added = 0;
+	EList<TIndexOffU> sorted;
 	sorted.resizeExact(_isaPrime.size());
-	sorted.fill(0xffffffff);
+	sorted.fill(OFF_MASK);
 	for(size_t di = 0; di < this->d(); di++) {
 		uint32_t d = _ds[di];
 		size_t i = 0;
 		for(size_t doi = _doffs[di]; doi < _doffs[di+1]; doi++, i++) {
-			assert_eq(0xffffffff, sorted[_isaPrime[doi]]);
+			assert_eq(OFF_MASK, sorted[_isaPrime[doi]]);
 			// Maps the offset of the suffix to its rank
-			sorted[_isaPrime[doi]] = (uint32_t)(v*i + d);
+			sorted[_isaPrime[doi]] = (TIndexOffU)(v*i + d);
 			added++;
 		}
 	}
@@ -591,25 +597,25 @@ void DifferenceCoverSample<TStr>::doBuiltSanityCheck() const {
  */
 template <typename TStr>
 void DifferenceCoverSample<TStr>::buildSPrime(
-	EList<uint32_t>& sPrime,
+	EList<TIndexOffU>& sPrime,
 	size_t padding)
 {
 	const TStr& t = this->text();
 	const EList<uint32_t>& ds = this->ds();
-	uint32_t tlen = (uint32_t)t.length();
+	TIndexOffU tlen = (TIndexOffU)t.length();
 	uint32_t v = this->v();
 	uint32_t d = this->d();
 	assert_gt(v, 2);
 	assert_lt(d, v);
 	// Record where each d section should begin in sPrime
-	uint32_t tlenDivV = this->divv(tlen);
+	TIndexOffU tlenDivV = this->divv(tlen);
 	uint32_t tlenModV = this->modv(tlen);
-	uint32_t sPrimeSz = 0;
+	TIndexOffU sPrimeSz = 0;
 	assert(_doffs.empty());
 	_doffs.resizeExact((size_t)d+1);
 	for(uint32_t di = 0; di < d; di++) {
 		// mu mapping
-		uint32_t sz = tlenDivV + ((ds[di] <= tlenModV) ? 1 : 0);
+		TIndexOffU sz = tlenDivV + ((ds[di] <= tlenModV) ? 1 : 0);
 		assert_geq(sz, 0);
 		_doffs[di] = sPrimeSz;
 		sPrimeSz += sz;
@@ -619,7 +625,7 @@ void DifferenceCoverSample<TStr>::buildSPrime(
 	if(tlenDivV > 0) {
 		for(size_t i = 0; i < d; i++) {
 			assert_gt(_doffs[i+1], _doffs[i]);
-			uint32_t diff = _doffs[i+1] - _doffs[i];
+			TIndexOffU diff = _doffs[i+1] - _doffs[i];
 			assert(diff == tlenDivV || diff == tlenDivV+1);
 		}
 	}
@@ -627,20 +633,20 @@ void DifferenceCoverSample<TStr>::buildSPrime(
 	assert_eq(_doffs.size(), d+1);
 	// Size sPrime appropriately
 	sPrime.resizeExact((size_t)sPrimeSz + padding);
-	sPrime.fill(0xffffffff);
+	sPrime.fill(OFF_MASK);
 	// Slot suffixes from text into sPrime according to the mu
 	// mapping; where the mapping would leave a blank, insert a 0
-	uint32_t added = 0;
-	uint32_t i = 0;
-	for(uint32_t ti = 0; ti <= tlen; ti += v) {
+	TIndexOffU added = 0;
+	TIndexOffU i = 0;
+	for(TIndexOffU ti = 0; ti <= tlen; ti += v) {
 		for(uint32_t di = 0; di < d; di++) {
-			uint32_t tti = ti + ds[di];
+			TIndexOffU tti = ti + ds[di];
 			if(tti > tlen) break;
-			uint32_t spi = _doffs[di] + i;
+			TIndexOffU spi = _doffs[di] + i;
 			assert_lt(spi, _doffs[di+1]);
 			assert_leq(tti, tlen);
 			assert_lt(spi, sPrimeSz);
-			assert_eq(0xffffffff, sPrime[spi]);
+			assert_eq(OFF_MASK, sPrime[spi]);
 			sPrime[spi] = tti; added++;
 		}
 		i++;
@@ -655,11 +661,11 @@ void DifferenceCoverSample<TStr>::buildSPrime(
 template <typename TStr>
 static inline bool suffixSameUpTo(
 	const TStr& host,
-	uint32_t suf1,
-	uint32_t suf2,
-	uint32_t v)
+	TIndexOffU suf1,
+	TIndexOffU suf2,
+	TIndexOffU v)
 {
-	for(uint32_t i = 0; i < v; i++) {
+	for(TIndexOffU i = 0; i < v; i++) {
 		bool endSuf1 = suf1+i >= host.length();
 		bool endSuf2 = suf2+i >= host.length();
 		if((endSuf1 && !endSuf2) || (!endSuf1 && endSuf2)) return false;
@@ -669,126 +675,245 @@ static inline bool suffixSameUpTo(
 	return true;
 }
 
+template<typename TStr>
+struct VSortingParam {
+    DifferenceCoverSample<TStr>* dcs;
+    TIndexOffU*                  sPrimeArr;
+    size_t                       sPrimeSz;
+    TIndexOffU*                  sPrimeOrderArr;
+    size_t                       depth;
+    const EList<size_t>*         boundaries;
+    size_t*                      cur;
+    MUTEX_T*                     mutex;
+};
+
+template<typename TStr>
+#ifdef WITH_TBB
+class VSorting_worker {
+        void *vp;
+
+public:
+
+	VSorting_worker(const VSorting_worker& W): vp(W.vp) {};
+	VSorting_worker(void *vp_):vp(vp_) {};
+	void operator()() const
+	{
+#else
+static void VSorting_worker(void *vp)
+{
+#endif
+    VSortingParam<TStr>* param = (VSortingParam<TStr>*)vp;
+    DifferenceCoverSample<TStr>* dcs = param->dcs;
+    const TStr& host = dcs->text();
+    const size_t hlen = host.length();
+    uint32_t v = dcs->v();
+    while(true) {
+        size_t cur = 0;
+        {
+            ThreadSafe ts(param->mutex, true);
+            cur = *(param->cur);
+            (*param->cur)++;
+        }
+        if(cur >= param->boundaries->size()) return;
+        size_t begin = (cur == 0 ? 0 : (*param->boundaries)[cur-1]);
+        size_t end = (*param->boundaries)[cur];
+        assert_leq(begin, end);
+        if(end - begin <= 1) continue;
+        mkeyQSortSuf2(
+                      host,
+                      hlen,
+                      param->sPrimeArr,
+                      param->sPrimeSz,
+                      param->sPrimeOrderArr,
+                      4,
+                      begin,
+                      end,
+                      param->depth,
+                      v);
+    }
+}
+
+#ifdef WITH_TBB
+};
+#endif
+
 /**
  * Calculates a ranking of all suffixes in the sample and stores them,
  * packed according to the mu mapping, in _isaPrime.
  */
 template <typename TStr>
-void DifferenceCoverSample<TStr>::build() {
-	// Local names for relevant types
-	VMSG_NL("Building DifferenceCoverSample");
-	// Local names for relevant data
-	const TStr& t = this->text();
-	uint32_t v = this->v();
-	assert_gt(v, 2);
-	// Build s'
-	EList<uint32_t> sPrime;
-	// Need to allocate 2 extra elements at the end of the sPrime and _isaPrime
-	// arrays.  One element that's less than all others, and another that acts
-	// as needed padding for the Larsson-Sadakane sorting code.
-	size_t padding = 1;
-	VMSG_NL("  Building sPrime");
-	buildSPrime(sPrime, padding);
-	size_t sPrimeSz = sPrime.size() - padding;
-	assert_gt(sPrime.size(), padding);
-	assert_leq(sPrime.size(), t.length() + padding + 1);
-	uint32_t nextRank = 0;
-	{
-		VMSG_NL("  Building sPrimeOrder");
-		EList<uint32_t> sPrimeOrder;
-		sPrimeOrder.resizeExact(sPrimeSz);
-		for(uint32_t i = 0; i < sPrimeSz; i++) {
-			sPrimeOrder[i] = i;
-		}
-		// sPrime now holds suffix-offsets for DC samples.
-		{
-			Timer timer(cout, "  V-Sorting samples time: ", this->verbose());
-			VMSG_NL("  V-Sorting samples");
-			// Extract backing-store array from sPrime and sPrimeOrder;
-			// the mkeyQSortSuf2 routine works on the array for maximum
-			// efficiency
-			uint32_t *sPrimeArr = (uint32_t*)sPrime.ptr();
-			assert_eq(sPrimeArr[0], sPrime[0]);
-			assert_eq(sPrimeArr[sPrimeSz-1], sPrime[sPrimeSz-1]);
-			uint32_t *sPrimeOrderArr = (uint32_t*)sPrimeOrder.ptr();
-			assert_eq(sPrimeOrderArr[0], sPrimeOrder[0]);
-			assert_eq(sPrimeOrderArr[sPrimeSz-1], sPrimeOrder[sPrimeSz-1]);
-			// Sort sample suffixes up to the vth character using a
-			// multikey quicksort.  Sort time is proportional to the
-			// number of samples times v.  It isn't quadratic.
-			// sPrimeOrder is passed in as a swapping partner for
-			// sPrimeArr, i.e., every time the multikey qsort swaps
-			// elements in sPrime, it swaps the same elements in
-			// sPrimeOrder too.  This allows us to easily reconstruct
-			// what the sort did.
-			mkeyQSortSuf2(t, sPrimeArr, sPrimeSz, sPrimeOrderArr, 4,
-			              this->verbose(), this->sanityCheck(), v);
-			// Make sure sPrime and sPrimeOrder are consistent with
-			// their respective backing-store arrays
-			assert_eq(sPrimeArr[0], sPrime[0]);
-			assert_eq(sPrimeArr[sPrimeSz-1], sPrime[sPrimeSz-1]);
-			assert_eq(sPrimeOrderArr[0], sPrimeOrder[0]);
-			assert_eq(sPrimeOrderArr[sPrimeSz-1], sPrimeOrder[sPrimeSz-1]);
-		}
-		// Now assign the ranking implied by the sorted sPrime/sPrimeOrder
-		// arrays back into sPrime.
-		VMSG_NL("  Allocating rank array");
-		_isaPrime.resizeExact(sPrime.size());
-		ASSERT_ONLY(_isaPrime.fill(0xffffffff));
-		assert_gt(_isaPrime.size(), 0);
-		{
-			Timer timer(cout, "  Ranking v-sort output time: ", this->verbose());
-			VMSG_NL("  Ranking v-sort output");
-			for(size_t i = 0; i < sPrimeSz-1; i++) {
-				// Place the appropriate ranking
-				_isaPrime[sPrimeOrder[i]] = nextRank;
-				// If sPrime[i] and sPrime[i+1] are identical up to v, then we
-				// should give the next suffix the same rank
-				if(!suffixSameUpTo(t, sPrime[i], sPrime[i+1], v)) nextRank++;
-			}
-			_isaPrime[sPrimeOrder[sPrimeSz-1]] = nextRank; // finish off
-#ifndef NDEBUG
-			for(size_t i = 0; i < sPrimeSz; i++) {
-				assert_neq(0xffffffff, _isaPrime[i]);
-				assert_lt(_isaPrime[i], sPrimeSz);
-			}
+void DifferenceCoverSample<TStr>::build(int nthreads) {
+    // Local names for relevant types
+    VMSG_NL("Building DifferenceCoverSample");
+    // Local names for relevant data
+    const TStr& t = this->text();
+    uint32_t v = this->v();
+    assert_gt(v, 2);
+    // Build s'
+    EList<TIndexOffU> sPrime;
+    // Need to allocate 2 extra elements at the end of the sPrime and _isaPrime
+    // arrays.  One element that's less than all others, and another that acts
+    // as needed padding for the Larsson-Sadakane sorting code.
+    size_t padding = 1;
+    VMSG_NL("  Building sPrime");
+    buildSPrime(sPrime, padding);
+    size_t sPrimeSz = sPrime.size() - padding;
+    assert_gt(sPrime.size(), padding);
+    assert_leq(sPrime.size(), t.length() + padding + 1);
+    TIndexOffU nextRank = 0;
+    {
+        VMSG_NL("  Building sPrimeOrder");
+        EList<TIndexOffU> sPrimeOrder;
+        sPrimeOrder.resizeExact(sPrimeSz);
+        for(TIndexOffU i = 0; i < sPrimeSz; i++) {
+            sPrimeOrder[i] = i;
+        }
+        // sPrime now holds suffix-offsets for DC samples.
+        {
+            Timer timer(cout, "  V-Sorting samples time: ", this->verbose());
+            VMSG_NL("  V-Sorting samples");
+            // Extract backing-store array from sPrime and sPrimeOrder;
+            // the mkeyQSortSuf2 routine works on the array for maximum
+            // efficiency
+            TIndexOffU *sPrimeArr = (TIndexOffU*)sPrime.ptr();
+            assert_eq(sPrimeArr[0], sPrime[0]);
+            assert_eq(sPrimeArr[sPrimeSz-1], sPrime[sPrimeSz-1]);
+            TIndexOffU *sPrimeOrderArr = (TIndexOffU*)sPrimeOrder.ptr();
+            assert_eq(sPrimeOrderArr[0], sPrimeOrder[0]);
+            assert_eq(sPrimeOrderArr[sPrimeSz-1], sPrimeOrder[sPrimeSz-1]);
+            // Sort sample suffixes up to the vth character using a
+            // multikey quicksort.  Sort time is proportional to the
+            // number of samples times v.  It isn't quadratic.
+            // sPrimeOrder is passed in as a swapping partner for
+            // sPrimeArr, i.e., every time the multikey qsort swaps
+            // elements in sPrime, it swaps the same elements in
+            // sPrimeOrder too.  This allows us to easily reconstruct
+            // what the sort did.
+            if(nthreads == 1) {
+                mkeyQSortSuf2(t, sPrimeArr, sPrimeSz, sPrimeOrderArr, 4,
+                              this->verbose(), this->sanityCheck(), v);
+            } else {
+                int query_depth = 0;
+                int tmp_nthreads = nthreads;
+                while(tmp_nthreads > 0) {
+                    query_depth++;
+                    tmp_nthreads >>= 1;
+                }
+                EList<size_t> boundaries; // bucket boundaries for parallelization
+                TIndexOffU *sOrig = NULL;
+                if(this->sanityCheck()) {
+                    sOrig = new TIndexOffU[sPrimeSz];
+                    memcpy(sOrig, sPrimeArr, OFF_SIZE * sPrimeSz);
+                }
+                mkeyQSortSuf2(t, sPrimeArr, sPrimeSz, sPrimeOrderArr, 4,
+                              this->verbose(), false, query_depth, &boundaries);
+                if(boundaries.size() > 0) {
+#ifdef WITH_TBB
+		    tbb::task_group tbb_grp;
+#else
+                    AutoArray<tthread::thread*> threads(nthreads);
 #endif
-		}
-		// sPrimeOrder is destroyed
-		// All the information we need is now in _isaPrime
-	}
-	_isaPrime[_isaPrime.size()-1] = (uint32_t)sPrimeSz;
-	sPrime[sPrime.size()-1] = (uint32_t)sPrimeSz;
-	// _isaPrime[_isaPrime.size()-1] and sPrime[sPrime.size()-1] are just
-	// spacer for the Larsson-Sadakane routine to use
-	{
-		Timer timer(cout, "  Invoking Larsson-Sadakane on ranks time: ", this->verbose());
-		VMSG_NL("  Invoking Larsson-Sadakane on ranks");
-		if(sPrime.size() >= 0x10000000) {
-			cerr << "Error; sPrime array has so many elements that it can't be converted to a signed array without overflow." << endl;
-			throw 1;
-		}
-		LarssonSadakane<int> ls;
-		ls.suffixsort(
-			(int*)_isaPrime.ptr(),
-			(int*)sPrime.ptr(),
-			(int)sPrimeSz,
-			(int)sPrime.size(),
-			0);
-	}
-	// chop off final character of _isaPrime
-	_isaPrime.resizeExact(sPrimeSz);
-	for(size_t i = 0; i < _isaPrime.size(); i++) {
-		_isaPrime[i]--;
-	}
-#ifndef NDEBUG
-	for(size_t i = 0; i < sPrimeSz-1; i++) {
-		assert_lt(_isaPrime[i], sPrimeSz);
-		assert(i == 0 || _isaPrime[i] != _isaPrime[i-1]);
-	}
+                    EList<VSortingParam<TStr> > tparams;
+                    size_t cur = 0;
+                    MUTEX_T mutex;
+                    tparams.resize(nthreads);
+                    for(int tid = 0; tid < nthreads; tid++) {
+                        // Calculate bucket sizes by doing a binary search for each
+                        // suffix and noting where it lands
+                        tparams[tid].dcs = this;
+                        tparams[tid].sPrimeArr = sPrimeArr;
+                        tparams[tid].sPrimeSz = sPrimeSz;
+                        tparams[tid].sPrimeOrderArr = sPrimeOrderArr;
+                        tparams[tid].depth = query_depth;
+                        tparams[tid].boundaries = &boundaries;
+                        tparams[tid].cur = &cur;
+                        tparams[tid].mutex = &mutex;
+#ifdef WITH_TBB
+			tbb_grp.run(VSorting_worker<TStr>(((void*)&tparams[tid])));
+		    }
+		    tbb_grp.wait();
+#else
+                        threads[tid] = new tthread::thread(VSorting_worker<TStr>, (void*)&tparams[tid]);
+                    }
+                    for (int tid = 0; tid < nthreads; tid++) {
+                        threads[tid]->join();
+                    }
 #endif
-	VMSG_NL("  Sanity-checking and returning");
-	if(this->sanityCheck()) doBuiltSanityCheck();
+                }
+                if(this->sanityCheck()) {
+                    sanityCheckOrderedSufs(t, t.length(), sPrimeArr, sPrimeSz, v);
+                    for(size_t i = 0; i < sPrimeSz; i++) {
+                        assert_eq(sPrimeArr[i], sOrig[sPrimeOrderArr[i]]);
+                    }
+                    delete[] sOrig;
+                }
+            }
+            // Make sure sPrime and sPrimeOrder are consistent with
+            // their respective backing-store arrays
+            assert_eq(sPrimeArr[0], sPrime[0]);
+            assert_eq(sPrimeArr[sPrimeSz-1], sPrime[sPrimeSz-1]);
+            assert_eq(sPrimeOrderArr[0], sPrimeOrder[0]);
+            assert_eq(sPrimeOrderArr[sPrimeSz-1], sPrimeOrder[sPrimeSz-1]);
+        }
+        // Now assign the ranking implied by the sorted sPrime/sPrimeOrder
+        // arrays back into sPrime.
+        VMSG_NL("  Allocating rank array");
+        _isaPrime.resizeExact(sPrime.size());
+        ASSERT_ONLY(_isaPrime.fill(OFF_MASK));
+        assert_gt(_isaPrime.size(), 0);
+        {
+            Timer timer(cout, "  Ranking v-sort output time: ", this->verbose());
+            VMSG_NL("  Ranking v-sort output");
+            for(size_t i = 0; i < sPrimeSz-1; i++) {
+                // Place the appropriate ranking
+                _isaPrime[sPrimeOrder[i]] = nextRank;
+                // If sPrime[i] and sPrime[i+1] are identical up to v, then we
+                // should give the next suffix the same rank
+                if(!suffixSameUpTo(t, sPrime[i], sPrime[i+1], v)) nextRank++;
+            }
+            _isaPrime[sPrimeOrder[sPrimeSz-1]] = nextRank; // finish off
+#ifndef NDEBUG
+            for(size_t i = 0; i < sPrimeSz; i++) {
+                assert_neq(OFF_MASK, _isaPrime[i]);
+                assert_lt(_isaPrime[i], sPrimeSz);
+            }
+#endif
+        }
+        // sPrimeOrder is destroyed
+        // All the information we need is now in _isaPrime
+    }
+    _isaPrime[_isaPrime.size()-1] = (TIndexOffU)sPrimeSz;
+    sPrime[sPrime.size()-1] = (TIndexOffU)sPrimeSz;
+    // _isaPrime[_isaPrime.size()-1] and sPrime[sPrime.size()-1] are just
+    // spacer for the Larsson-Sadakane routine to use
+    {
+        Timer timer(cout, "  Invoking Larsson-Sadakane on ranks time: ", this->verbose());
+        VMSG_NL("  Invoking Larsson-Sadakane on ranks");
+        if(sPrime.size() >= LS_SIZE) {
+            cerr << "Error; sPrime array has so many elements that it can't be converted to a signed array without overflow." << endl;
+            throw 1;
+        }
+        LarssonSadakane<TIndexOff> ls;
+        ls.suffixsort(
+                      (TIndexOff*)_isaPrime.ptr(),
+                      (TIndexOff*)sPrime.ptr(),
+                      (TIndexOff)sPrimeSz,
+                      (TIndexOff)sPrime.size(),
+                      0);
+    }
+    // chop off final character of _isaPrime
+    _isaPrime.resizeExact(sPrimeSz);
+    for(size_t i = 0; i < _isaPrime.size(); i++) {
+        _isaPrime[i]--;
+    }
+#ifndef NDEBUG
+    for(size_t i = 0; i < sPrimeSz-1; i++) {
+        assert_lt(_isaPrime[i], sPrimeSz);
+        assert(i == 0 || _isaPrime[i] != _isaPrime[i-1]);
+    }
+#endif
+    VMSG_NL("  Sanity-checking and returning");
+    if(this->sanityCheck()) doBuiltSanityCheck();
 }
 
 /**
@@ -797,7 +922,7 @@ void DifferenceCoverSample<TStr>::build() {
  * logic elsewhere.
  */
 template <typename TStr>
-bool DifferenceCoverSample<TStr>::isCovered(uint32_t i) const {
+bool DifferenceCoverSample<TStr>::isCovered(TIndexOffU i) const {
 	assert(built());
 	uint32_t modi = this->modv(i);
 	assert_lt(modi, _dInv.size());
@@ -809,16 +934,16 @@ bool DifferenceCoverSample<TStr>::isCovered(uint32_t i) const {
  * among the sample suffixes.
  */
 template <typename TStr>
-uint32_t DifferenceCoverSample<TStr>::rank(uint32_t i) const {
+TIndexOffU DifferenceCoverSample<TStr>::rank(TIndexOffU i) const {
 	assert(built());
 	assert_lt(i, this->text().length());
 	uint32_t imodv = this->modv(i);
 	assert_neq(0xffffffff, _dInv[imodv]); // must be in the sample
-	uint32_t ioff = this->divv(i);
+	TIndexOffU ioff = this->divv(i);
 	assert_lt(ioff, _doffs[_dInv[imodv]+1] - _doffs[_dInv[imodv]]);
-	uint32_t isaIIdx = _doffs[_dInv[imodv]] + ioff;
+	TIndexOffU isaIIdx = _doffs[_dInv[imodv]] + ioff;
 	assert_lt(isaIIdx, _isaPrime.size());
-	uint32_t isaPrimeI = _isaPrime[isaIIdx];
+	TIndexOffU isaPrimeI = _isaPrime[isaIIdx];
 	assert_leq(isaPrimeI, _isaPrime.size());
 	return isaPrimeI;
 }
@@ -828,7 +953,7 @@ uint32_t DifferenceCoverSample<TStr>::rank(uint32_t i) const {
  * if suffix j is lexicographically greater.
  */
 template <typename TStr>
-int64_t DifferenceCoverSample<TStr>::breakTie(uint32_t i, uint32_t j) const {
+int64_t DifferenceCoverSample<TStr>::breakTie(TIndexOffU i, TIndexOffU j) const {
 	assert(built());
 	assert_neq(i, j);
 	assert_lt(i, this->text().length());
@@ -839,20 +964,20 @@ int64_t DifferenceCoverSample<TStr>::breakTie(uint32_t i, uint32_t j) const {
 	assert_neq(0xffffffff, _dInv[jmodv]); // must be in the sample
 	uint32_t dimodv = _dInv[imodv];
 	uint32_t djmodv = _dInv[jmodv];
-	uint32_t ioff = this->divv(i);
-	uint32_t joff = this->divv(j);
+	TIndexOffU ioff = this->divv(i);
+	TIndexOffU joff = this->divv(j);
 	assert_lt(dimodv+1, _doffs.size());
 	assert_lt(djmodv+1, _doffs.size());
 	// assert_lt: expected (32024) < (0)
 	assert_lt(ioff, _doffs[dimodv+1] - _doffs[dimodv]);
 	assert_lt(joff, _doffs[djmodv+1] - _doffs[djmodv]);
-	uint32_t isaIIdx = _doffs[dimodv] + ioff;
-	uint32_t isaJIdx = _doffs[djmodv] + joff;
+	TIndexOffU isaIIdx = _doffs[dimodv] + ioff;
+	TIndexOffU isaJIdx = _doffs[djmodv] + joff;
 	assert_lt(isaIIdx, _isaPrime.size());
 	assert_lt(isaJIdx, _isaPrime.size());
 	assert_neq(isaIIdx, isaJIdx); // ranks must be unique
-	uint32_t isaPrimeI = _isaPrime[isaIIdx];
-	uint32_t isaPrimeJ = _isaPrime[isaJIdx];
+	TIndexOffU isaPrimeI = _isaPrime[isaIIdx];
+	TIndexOffU isaPrimeJ = _isaPrime[isaJIdx];
 	assert_neq(isaPrimeI, isaPrimeJ); // ranks must be unique
 	assert_leq(isaPrimeI, _isaPrime.size());
 	assert_leq(isaPrimeJ, _isaPrime.size());
@@ -864,7 +989,7 @@ int64_t DifferenceCoverSample<TStr>::breakTie(uint32_t i, uint32_t j) const {
  * be compared before the difference cover can break the tie.
  */
 template <typename TStr>
-uint32_t DifferenceCoverSample<TStr>::tieBreakOff(uint32_t i, uint32_t j) const {
+uint32_t DifferenceCoverSample<TStr>::tieBreakOff(TIndexOffU i, TIndexOffU j) const {
 	const TStr& t = this->text();
 	const EList<uint32_t>& dmap = this->dmap();
 	assert(built());
