@@ -29,6 +29,7 @@
 #include "assert_helpers.h"
 #include "threading.h"
 #include "random_source.h"
+#include "btypes.h"
 
 /**
  * Tally how much memory is allocated to certain 
@@ -98,7 +99,9 @@ protected:
 	uint64_t peak_;
 };
 
+#ifdef USE_MEM_TALLY
 extern MemoryTally gMemTally;
+#endif
 
 /**
  * A simple fixed-length array of type T, automatically freed in the
@@ -111,7 +114,11 @@ public:
 	AutoArray(size_t sz, int cat = 0) : cat_(cat) {
 		t_ = NULL;
 		t_ = new T[sz];
+#ifdef USE_MEM_TALLY
 		gMemTally.add(cat_, sz);
+#else
+		(void)cat_;
+#endif
 		memset(t_, 0, sz * sizeof(T));
 		sz_ = sz;
 	}
@@ -119,7 +126,9 @@ public:
 	~AutoArray() {
 		if(t_ != NULL) {
 			delete[] t_;
+#ifdef USE_MEM_TALLY
 			gMemTally.del(cat_, sz_);
+#endif
 		}
 	}
 	
@@ -176,16 +185,22 @@ public:
 		assert(p_ == NULL);
 		p_ = p;
 		freeable_ = freeable;
+#ifdef USE_MEM_TALLY
 		if(p != NULL && freeable_) {
 			gMemTally.add(cat_, sizeof(T));
 		}
+#else
+		(void)cat_;
+#endif
 	}
 	
 	void free() {
 		if(p_ != NULL) {
 			if(freeable_) {
 				delete p_;
+#ifdef USE_MEM_TALLY
 				gMemTally.del(cat_, sizeof(T));
+#endif
 			}
 			p_ = NULL;
 		}
@@ -239,16 +254,22 @@ public:
 		p_ = p;
 		sz_ = sz;
 		freeable_ = freeable;
+#ifdef USE_MEM_TALLY
 		if(p != NULL && freeable_) {
 			gMemTally.add(cat_, sizeof(T) * sz_);
 		}
+#else
+		(void)cat_;
+#endif
 	}
 	
 	void free() {
 		if(p_ != NULL) {
 			if(freeable_) {
 				delete[] p_;
+#ifdef USE_MEM_TALLY
 				gMemTally.del(cat_, sizeof(T) * sz_);
+#endif
 			}
 			p_ = NULL;
 		}
@@ -318,7 +339,12 @@ public:
 	 * Allocate initial default of S elements.
 	 */
 	explicit EList() :
-		cat_(0), allocCat_(-1), list_(NULL), sz_(S), cur_(0) { }
+		cat_(0), allocCat_(-1), list_(NULL), sz_(S), cur_(0)
+	{
+#ifndef USE_MEM_TALLY
+		(void)cat_;
+#endif
+	}
 
 	/**
 	 * Allocate initial default of S elements.
@@ -769,7 +795,7 @@ public:
 	void sortPortion(size_t begin, size_t num) {
 		assert_leq(begin+num, cur_);
 		if(num < 2) return;
-		std::sort(list_ + begin, list_ + begin + num);
+		std::stable_sort(list_ + begin, list_ + begin + num);
 	}
 	
 	/**
@@ -780,7 +806,7 @@ public:
 		if(num < 2) return;
 		size_t left = num;
 		for(size_t i = begin; i < begin + num - 1; i++) {
-			uint32_t rndi = rnd.nextU32() % left;
+			size_t rndi = rnd.nextSizeT() % left;
 			if(rndi > 0) {
 				std::swap(list_[i], list_[i + rndi]);
 			}
@@ -896,7 +922,9 @@ private:
 	T *alloc(size_t sz) {
 		T* tmp = new T[sz];
 		assert(tmp != NULL);
+#ifdef USE_MEM_TALLY
 		gMemTally.add(cat_, sz);
+#endif
 		allocCat_ = cat_;
 		return tmp;
 	}
@@ -910,7 +938,9 @@ private:
 			assert_neq(-1, allocCat_);
 			assert_eq(allocCat_, cat_);
 			delete[] list_;
+#ifdef USE_MEM_TALLY
 			gMemTally.del(cat_, sz_);
+#endif
 			list_ = NULL;
 			sz_ = cur_ = 0;
 		}
@@ -1014,6 +1044,9 @@ public:
 	explicit ELList(int cat = 0) :
 		cat_(cat), list_(NULL), sz_(S2), cur_(0)
 	{
+#ifndef USE_MEM_TALLY
+		(void)cat_;
+#endif
 		assert_geq(cat, 0);
 	}
 
@@ -1255,7 +1288,9 @@ protected:
 	EList<T, S1> *alloc(size_t sz) {
 		assert_gt(sz, 0);
 		EList<T, S1> *tmp = new EList<T, S1>[sz];
+#ifdef USE_MEM_TALLY
 		gMemTally.add(cat_, sz);
+#endif
 		if(cat_ != 0) {
 			for(size_t i = 0; i < sz; i++) {
 				assert(tmp[i].ptr() == NULL);
@@ -1272,7 +1307,9 @@ protected:
 	void free() {
 		if(list_ != NULL) {
 			delete[] list_;
+#ifdef USE_MEM_TALLY
 			gMemTally.del(cat_, sz_);
+#endif
 			list_ = NULL;
 		}
 	}
@@ -1591,7 +1628,9 @@ protected:
 	ELList<T, S1, S2> *alloc(size_t sz) {
 		assert_gt(sz, 0);
 		ELList<T, S1, S2> *tmp = new ELList<T, S1, S2>[sz];
+#ifdef USE_MEM_TALLY
 		gMemTally.add(cat_, sz);
+#endif
 		if(cat_ != 0) {
 			for(size_t i = 0; i < sz; i++) {
 				assert(tmp[i].ptr() == NULL);
@@ -1608,7 +1647,9 @@ protected:
 	void free() {
 		if(list_ != NULL) {
 			delete[] list_;
+#ifdef USE_MEM_TALLY
 			gMemTally.del(cat_, sz_);
+#endif
 			list_ = NULL;
 		}
 	}
@@ -1678,6 +1719,9 @@ public:
 		sz_(0),
 		cur_(0)
 	{
+#ifndef USE_MEM_TALLY
+		(void)cat_;
+#endif
 		if(sz_ > 0) {
 			list_ = alloc(sz_);
 		}
@@ -1879,7 +1923,9 @@ private:
 	T *alloc(size_t sz) {
 		assert_gt(sz, 0);
 		T *tmp = new T[sz];
+#ifdef USE_MEM_TALLY
 		gMemTally.add(cat_, sz);
+#endif
 		return tmp;
 	}
 
@@ -1890,7 +1936,9 @@ private:
 	void free() {
 		if(list_ != NULL) {
 			delete[] list_;
+#ifdef USE_MEM_TALLY
 			gMemTally.del(cat_, sz_);
+#endif
 			list_ = NULL;
 		}
 	}
@@ -2025,6 +2073,9 @@ public:
 	explicit ELSet(int cat = 0) :
 		cat_(cat), list_(NULL), sz_(S), cur_(0)
 	{
+#ifndef USE_MEM_TALLY
+		(void)cat_;
+#endif
 		assert_geq(cat, 0);
 	}
 
@@ -2271,7 +2322,9 @@ protected:
 	ESet<T> *alloc(size_t sz) {
 		assert_gt(sz, 0);
 		ESet<T> *tmp = new ESet<T>[sz];
+#ifdef USE_MEM_TALLY
 		gMemTally.add(cat_, sz);
+#endif
 		if(cat_ != 0) {
 			for(size_t i = 0; i < sz; i++) {
 				assert(tmp[i].ptr() == NULL);
@@ -2288,7 +2341,9 @@ protected:
 	void free() {
 		if(list_ != NULL) {
 			delete[] list_;
+#ifdef USE_MEM_TALLY
 			gMemTally.del(cat_, sz_);
+#endif
 			list_ = NULL;
 		}
 	}
@@ -2359,6 +2414,9 @@ public:
 		sz_(128),
 		cur_(0)
 	{
+#ifndef USE_MEM_TALLY
+		(void)cat_;
+#endif
 		list_ = alloc(sz_);
 	}
 
@@ -2538,7 +2596,9 @@ private:
 	std::pair<K, V> *alloc(size_t sz) {
 		assert_gt(sz, 0);
 		std::pair<K, V> *tmp = new std::pair<K, V>[sz];
+#ifdef USE_MEM_TALLY
 		gMemTally.add(cat_, sz);
+#endif
 		return tmp;
 	}
 
@@ -2549,7 +2609,9 @@ private:
 	void free() {
 		if(list_ != NULL) {
 			delete[] list_;
+#ifdef USE_MEM_TALLY
 			gMemTally.del(cat_, sz_);
+#endif
 			list_ = NULL;
 		}
 	}
@@ -2970,10 +3032,10 @@ public:
         size_t c1 = ((cur+1) << 1) - 1;
         size_t c2 = c1 + 1;
 		if(c1 < l_.size()) {
-			assert_leq(l_[cur], l_[c1]);
+			assert(l_[cur] <= l_[c1]);
 		}
 		if(c2 < l_.size()) {
-			assert_leq(l_[cur], l_[c2]);
+			assert(l_[cur] <= l_[c2]);
 		}
 		if(c2 < l_.size()) {
 			return repOkNode(c1) && repOkNode(c2);
@@ -3009,13 +3071,17 @@ public:
 		int cat = 0) :
 		cat_(cat),
 		cur_(0),
-		bytes_(bytes),
 		pagesz_(pagesz),
 		pages_(cat)
 	{
 		for(size_t i = 0; i < ((bytes+pagesz-1)/pagesz); i++) {
 			pages_.push_back(new uint8_t[pagesz]);
+#ifdef USE_MEM_TALLY
 			gMemTally.add(cat, pagesz);
+#else
+			(void)cat_;
+			(void)pagesz_;
+#endif
 			assert(pages_.back() != NULL);
 		}
 		assert(repOk());
@@ -3028,7 +3094,11 @@ public:
 		for(size_t i = 0; i < pages_.size(); i++) {
 			assert(pages_[i] != NULL);
 			delete[] pages_[i];
+#ifdef USE_MEM_TALLY
 			gMemTally.del(cat_, pagesz_);
+#else
+			(void)cat_;
+#endif
 		}
 	}
 
@@ -3064,7 +3134,6 @@ public:
 	bool repOk() const {
 		assert_leq(cur_, pages_.size());
 		assert(!pages_.empty());
-		assert_gt(bytes_, 0);
 		assert_gt(pagesz_, 0);
 		return true;
 	}
@@ -3072,9 +3141,8 @@ public:
 
 private:
 	int             cat_;    // memory category, for accounting purposes
-	uint32_t        cur_;    // next page to hand out
-	const uint64_t  bytes_;  // total bytes in the pool
-	const uint32_t  pagesz_; // size of a single page
+	size_t          cur_;    // next page to hand out
+	const size_t    pagesz_; // size of a single page
 	EList<uint8_t*> pages_;  // the pages themselves
 };
 
@@ -3434,7 +3502,7 @@ public:
 	 * include elements that fall off the end of list_).
 	 */
 	void setLength(size_t nlen) {
-		len_ = (uint32_t)nlen;
+		len_ = nlen;
 	}
 	
 protected:
@@ -3458,8 +3526,8 @@ public:
 
 	PListSlice(
 		PList<T, S>& list,
-		uint32_t i,
-		uint32_t len) :
+		TIndexOffU i,
+		TIndexOffU len) :
 		i_(i),
 		len_(len),
 		list_(&list)
@@ -3471,8 +3539,8 @@ public:
 	void init(const PListSlice<T, S>& sl, size_t first, size_t last) {
 		assert_gt(last, first);
 		assert_leq(last - first, sl.len_);
-		i_ = (uint32_t)(sl.i_ + first);
-		len_ = (uint32_t)(last - first);
+		i_ = (TIndexOffU)(sl.i_ + first);
+		len_ = (TIndexOffU)(last - first);
 		list_ = sl.list_;
 	}
 	
@@ -3566,12 +3634,12 @@ public:
 	 * include elements that fall off the end of list_).
 	 */
 	void setLength(size_t nlen) {
-		len_ = (uint32_t)nlen;
+		len_ = (TIndexOffU)nlen;
 	}
 	
 protected:
-	uint32_t i_;
-	uint32_t len_;
+	TIndexOffU i_;
+	TIndexOffU len_;
 	PList<T, S>* list_;
 };
 
@@ -3698,7 +3766,7 @@ public:
 	 * Initialize the current-edit pointer to 0 and set the number of
 	 * edits per memory page.
 	 */
-	RedBlack(uint32_t pageSz, int cat = 0) :
+	RedBlack(size_t pageSz, int cat = 0) :
 		perPage_(pageSz/sizeof(TNode)), pages_(cat) { clear(); }
 
 	/**
