@@ -36,6 +36,7 @@ enum {
 	ARG_PARTITION,              // --partition
 	ARG_INTEGER_QUALS,          // --int-quals
 	ARG_FILEPAR,                // --filepar
+	ARG_READS_PER_BATCH,        // --reads-per-batch
 	ARG_SHMEM,                  // --shmem
 	ARG_MM,                     // --mm
 	ARG_MMSWEEP,                // --mmsweep
@@ -53,7 +54,6 @@ enum {
 	ARG_PHRED64,                // --phred64
 	ARG_PHRED33,                // --phred33
 	ARG_HADOOPOUT,              // --hadoopout
-	ARG_FUZZY,                  // --fuzzy
 	ARG_FULLREF,                // --fullref
 	ARG_USAGE,                  // --usage
 	ARG_SNPPHRED,               // --snpphred
@@ -97,7 +97,6 @@ enum {
 	ARG_PRESET_FAST_LOCAL,           // --fast-local
 	ARG_PRESET_SENSITIVE_LOCAL,      // --sensitive-local
 	ARG_PRESET_VERY_SENSITIVE_LOCAL, // --very-sensitive-local
-	ARG_NO_SCORE_PRIORITY,      // --no-score-priority
 	ARG_IGNORE_QUALS,           // --ignore-quals
 	ARG_DESC,                   // --arg-desc
 	ARG_TAB5,                   // --tab5
@@ -149,7 +148,10 @@ enum {
 	ARG_DESC_KB,                // --desc-kb
 	ARG_DESC_LANDING,           // --desc-landing
 	ARG_DESC_EXP,               // --desc-exp
-	ARG_DESC_FMOPS              // --desc-fmops
+	ARG_DESC_PRIORITIZE,        // --desc-prioritize
+	ARG_DESC_FMOPS,             // --desc-fmops
+	ARG_LOG_DP,                 // --log-dp
+	ARG_LOG_DP_OPP              // --log-dp-opp
 };
 
 #endif
