@@ -104,6 +104,9 @@ void SamConfig::printPgLine(BTString& o) const {
 	o.append(pg_pn_.c_str());
 	o.append("\tVN:");
 	o.append(pg_vn_.c_str());
+	o.append("\tCL:\"");
+	o.append(pg_cl_.c_str());
+	o.append('"');
 	o.append('\n');
 }
 
@@ -119,6 +122,7 @@ void SamConfig::printAlignedOptFlags(
 	BTString& o,               // output buffer
 	bool first,                // first opt flag printed is first overall?
 	const Read& rd,            // the read
+	const Read* rdo,           // the opposite read
 	AlnRes& res,               // individual alignment result
 	StackedAln& staln,         // stacked alignment buffer
 	const AlnFlags& flags,     // alignment flags
@@ -130,6 +134,7 @@ void SamConfig::printAlignedOptFlags(
 	const
 {
 	char buf[1024];
+	assert(summ.best(rd.mate < 2).valid());
 	if(print_as_) {
 		// AS:i: Alignment score generated by aligner
 		itoa10<TAlScore>(res.score().score(), buf);
@@ -139,7 +144,12 @@ void SamConfig::printAlignedOptFlags(
 	}
 	if(print_xs_) {
 		// XS:i: Suboptimal alignment score
-		AlnScore sco = summ.secbestMate(rd.mate < 2);
+		AlnScore sco;
+		if(flags.alignedConcordant()) {
+			sco = summ.bestUnchosen(rd.mate < 2);
+		} else {
+			sco = summ.secbest(rd.mate < 2);
+		}
 		if(sco.valid()) {
 			itoa10<TAlScore>(sco.score(), buf);
 			WRITE_SEP();
@@ -246,6 +256,21 @@ void SamConfig::printAlignedOptFlags(
 		WRITE_SEP();
 		o.append("Yn:i:");
 		o.append(buf);
+		if(summ.paired()) {
+			assert(rdo != NULL);
+			// ZN:i: Minimum valid score for opposite mate
+			TAlScore mn = sc.scoreMin.f<TAlScore>(rdo->length());
+			itoa10<TAlScore>(mn, buf);
+			WRITE_SEP();
+			o.append("ZN:i:");
+			o.append(buf);
+			// Zn:i: Perfect score for opposite mate
+			TAlScore pe = sc.perfectScore(rdo->length());
+			itoa10<TAlScore>(pe, buf);
+			WRITE_SEP();
+			o.append("Zn:i:");
+			o.append(buf);
+		}
 	}
 	if(print_xss_) {
 		// Xs:i: Best invalid alignment score of this mate
@@ -308,22 +333,18 @@ void SamConfig::printAlignedOptFlags(
 	}
 	if(flags.partOfPair() && print_zp_) {
 		// ZP:i: Score of best concordant paired-end alignment
-		WRITE_SEP();
-		o.append("ZP:Z:");
 		if(summ.bestPaired().valid()) {
+			WRITE_SEP();
+			o.append("ZP:i:");
 			itoa10<TAlScore>(summ.bestPaired().score(), buf);
 			o.append(buf);
-		} else {
-			o.append("NA");
 		}
-		// Zp:i: Second-best concordant paired-end alignment score
-		WRITE_SEP();
-		o.append("Zp:Z:");
+		// Zp:i: Score of second-best concordant paired-end alignment
 		if(summ.secbestPaired().valid()) {
+			WRITE_SEP();
+			o.append("Zp:i:");
 			itoa10<TAlScore>(summ.secbestPaired().score(), buf);
 			o.append(buf);
-		} else {
-			o.append("NA");
 		}
 	}
 	if(print_zu_) {
@@ -498,6 +519,110 @@ void SamConfig::printAlignedOptFlags(
 		o.append("\n");
 		printOptFieldNewlineEscapedZ(o, rd.readOrigBuf);
 	}
+	if(print_zt_) {
+		// ZT:Z: Extra features for MAPQ estimation
+		
+		// 1. AS:i for current mate
+		// 2. diff or NA for current mate
+		// 3. Like 1 for opposite mate
+		// 4. Like 2 for opposite mate
+		
+		WRITE_SEP();
+		const bool paired = flags.partOfPair();
+		const TAlScore MN = std::numeric_limits<TAlScore>::min();
+		TAlScore secondBest[2] = {MN, MN};
+		TAlScore thirdBest[2] = {MN, MN};
+		{
+			for (int self = 0; self < (paired ? 2 : 1); self++) {
+				// Second-best
+				AlnScore sco;
+				bool mate1 = rd.mate < 2;
+				if(self > 0) mate1 = !mate1;
+				if(flags.alignedConcordant()) {
+					sco = summ.bestUnchosen(mate1);
+				} else {
+					sco = summ.secbest(mate1);
+				}
+				if(sco.valid()) {
+					secondBest[self] = sco.score();
+				}
+
+				// Third-best
+				thirdBest[self] = mate1 ? prm.bestLtMinscMate1 : prm.bestLtMinscMate2;
+			}
+		}
+		TAlScore best[2] = {res.score().score(), res.oscore().score()};
+		TAlScore diff[2] = {MN, MN};
+		for(int self = 0; self < 2; self++) {
+			const TAlScore mx = max(secondBest[self], thirdBest[self]);
+			if(best[self] > MN && mx > MN) {
+				diff[self] = best[self] - mx;
+			}
+		}
+		TAlScore best_conc = MN, diff_conc = MN;
+		if(paired && summ.bestPaired().valid()) {
+			best_conc = summ.bestPaired().score();
+			if(summ.secbestPaired().valid()) {
+				diff_conc = best_conc - summ.secbestPaired().score();
+			}
+		}
+		o.append("ZT:Z:");
+		// AS:i for current mate
+		itoa10<TAlScore>((int)best[0], buf);
+		o.append(buf);
+		o.append(",");
+		// diff for current mate
+		if(diff[0] > MN) {
+			itoa10<TAlScore>((int)diff[0], buf);
+			o.append(buf);
+		} else {
+			o.append("NA");
+		}
+		o.append(",");
+		// AS:i for other mate
+		if(best[1] > MN) {
+			itoa10<TAlScore>((int)best[1], buf);
+			o.append(buf);
+		} else {
+			o.append("NA");
+		}
+		o.append(",");
+		// diff for other mate
+		if(diff[1] > MN) {
+			itoa10<TAlScore>((int)diff[1], buf);
+			o.append(buf);
+		} else {
+			o.append("NA");
+		}
+		o.append(",");
+		// Sum of AS:i for aligned pairs
+		if(best_conc > MN) {
+			itoa10<TAlScore>((int)best_conc, buf);
+			o.append(buf);
+		} else {
+			o.append("NA");
+		}
+		o.append(",");
+		// Diff for aligned pairs
+		if(diff_conc > MN) {
+			itoa10<TAlScore>((int)diff_conc, buf);
+			o.append(buf);
+		} else {
+			o.append("NA");
+		}
+		o.append(",");
+		itoa10<TAlScore>((int)(prm.seedsPerNuc * 1000), buf);
+		o.append(buf);
+		o.append(",");
+		itoa10<TAlScore>((int)(prm.seedPctUnique * 1000), buf);
+		o.append(buf);
+		o.append(",");
+		itoa10<TAlScore>((int)(prm.seedPctRep * 1000), buf);
+		o.append(buf);
+		o.append(",");
+		itoa10<TAlScore>((int)(prm.seedHitAvg + 0.5), buf);
+		o.append(buf);
+	}
 }
 
 /**
@@ -552,7 +677,7 @@ void SamConfig::printEmptyOptFlags(
 		flags.printYM(o);
 	}
 	if(print_yf_ && flags.filtered()) {
-		// YM:i: Read was repetitive when aligned unpaired?
+		// YF:i: Why read was filtered out prior to alignment
 		first = flags.printYF(o, first) && first;
 	}
 	if(!rgs_.empty()) {
